// Utils.hpp
#pragma once

#include <utility>
#include <algorithm>
#include <vector>
#include <cstdlib>

// ASSERT
#ifdef WIN32
	#include <crtdbg.h>
	#define ASSERT(exp)		_ASSERTE(exp)
#else
	#include <assert.h>
	#define ASSERT(exp)		assert(exp)
#endif


// deleter
class deleter
{
public:
	typedef void		result_type;
	template < typename T >
	void operator () (const T* ptr) const
	{
		delete ptr;
	}
};

// select1st
template < typename Operation > class selecter1st
{
protected:
	Operation	_op;
public:
	typedef typename Operation::result_type		result_type;

	selecter1st(const Operation& op) : _op(op)
	{}

	template < typename First, typename Second >
	result_type operator() (const std::pair<First, Second>& pair) const
	{
		return _op(pair.first);
	}
};
template < typename Operation > selecter1st<Operation>	select1st(const Operation& op)	{ return selecter1st<Operation>(op); }

// select2nd
template < typename Operation > class selecter2nd
{
protected:
	Operation	_op;
public:
	typedef typename Operation::result_type		result_type;

	selecter2nd(const Operation& op) : _op(op)
	{}

	template < typename First, typename Second >
	result_type operator() (const std::pair<First, Second>& pair) const
	{
		return _op(pair.second);
	}
};
template < typename Operation > selecter2nd<Operation>	select2nd(const Operation& op)	{ return selecter2nd<Operation>(op); }

// for_all
template < typename STLContainer, typename Function >
inline Function for_all(STLContainer& cont, Function func)
{
	return std::for_each(cont.begin(), cont.end(), func);
}

// auto_ptr_cast
template < typename To, typename From >
inline std::auto_ptr<To>	auto_ptr_cast(std::auto_ptr<From> ptr)
{
	return std::auto_ptr<To>(static_cast<To*>(ptr.release()));
}
template < typename T >
inline std::auto_ptr<T>	auto_ptr_cast(T* ptr)
{
	return std::auto_ptr<T>(ptr);
}

// mbs2wcs
class mbs2wcs
{
private:
	std::vector<wchar_t>	m_str;
public:
	explicit mbs2wcs(const char* src)			{ assign(src); }
	explicit mbs2wcs(const std::string& src)	{ assign(src); }
	void assign(const char* src)
	{
		m_str.resize(strlen(src) + 1);
		m_str.resize(mbstowcs(&m_str[0], src, m_str.size()) + 1);
	}
	void assign(const std::string& src)	{ assign(src.c_str()); }
	const wchar_t* str() const			{ return &m_str[0]; }
	operator const wchar_t* () const	{ return &m_str[0]; }
};

// wcs2mbs
class wcs2mbs
{
private:
	std::vector<char>	m_str;
public:
	explicit wcs2mbs(const wchar_t* src)		{ assign(src); }
	explicit wcs2mbs(const std::wstring& src)	{ assign(src); }
	void assign(const wchar_t* src)
	{
		m_str.resize(wcslen(src) * 2 + 1);
		m_str.resize(wcstombs(&m_str[0], src, m_str.size()) + 1);
	}
	void assign(const std::wstring& src)	{ assign(src.c_str()); }
	const char* str() const			{ return &m_str[0]; }
	operator const char* () const	{ return &m_str[0]; }
};

namespace str
{
	// less<T>
	template < typename T >	struct less;
	template <> struct less<char>
	{
		bool operator () (const char* lhs, const char* rhs) const
		{
			return strcmp(lhs, rhs) < 0;
		}
	};
	template <> struct less<wchar_t>
	{
		bool operator () (const wchar_t* lhs, const wchar_t* rhs) const
		{
			return wcscmp(lhs, rhs) < 0;
		}
	};
}

// VC++7.0 ł char  wchar_t Ԃ̕ϊftHgŃT|[gĂȂ߁B
inline std::ostream&  operator << (std::ostream&  os, const wcs2mbs& conv)		{ return os << conv.str();   }
inline std::ostream&  operator << (std::ostream&  os, const wchar_t* str)		{ return os << wcs2mbs(str); }
inline std::ostream&  operator << (std::ostream&  os, const std::wstring& str)	{ return os << wcs2mbs(str); }
inline std::wostream& operator << (std::wostream& os, const mbs2wcs& conv)		{ return os << conv.str();   }
inline std::wostream& operator << (std::wostream& os, const char* str)			{ return os << mbs2wcs(str); }
inline std::wostream& operator << (std::wostream& os, const std::string& str)	{ return os << mbs2wcs(str); }
